@@ -4,10 +4,6 @@
 #ifndef __INCLUDE_RTE_SWX_PIPELINE_H__
 #define __INCLUDE_RTE_SWX_PIPELINE_H__
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
 /**
  * @file
  * RTE SWX Pipeline
@@ -22,6 +18,10 @@ extern "C" {
 #include "rte_swx_table.h"
 #include "rte_swx_extern.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /** Name size. */
 #ifndef RTE_SWX_NAME_SIZE
 #define RTE_SWX_NAME_SIZE 64
@@ -44,22 +44,38 @@ extern "C" {
 /** Pipeline opaque data structure. */
 struct rte_swx_pipeline;
 
+/**
+ * Pipeline find
+ *
+ * @param[in] name
+ *   Pipeline name.
+ * @return
+ *   Valid pipeline handle if found or NULL otherwise.
+ */
+__rte_experimental
+struct rte_swx_pipeline *
+rte_swx_pipeline_find(const char *name);
+
 /**
  * Pipeline configure
  *
  * @param[out] p
  *   Pipeline handle. Must point to valid memory. Contains valid pipeline handle
  *   when the function returns successfully.
+ * @param[in] name
+ *   Pipeline unique name.
  * @param[in] numa_node
  *   Non-Uniform Memory Access (NUMA) node.
  * @return
  *   0 on success or the following error codes otherwise:
  *   -EINVAL: Invalid argument;
- *   -ENOMEM: Not enough space/cannot allocate memory.
+ *   -ENOMEM: Not enough space/cannot allocate memory;
+ *   -EEXIST: Pipeline with this name already exists.
  */
 __rte_experimental
 int
 rte_swx_pipeline_config(struct rte_swx_pipeline **p,
+			const char *name,
 			int numa_node);
 
 /*
@@ -315,23 +331,6 @@ rte_swx_pipeline_extern_func_register(struct rte_swx_pipeline *p,
  * Hash function.
  */
 
-/**
- * Hash function prototype
- *
- * @param[in] key
- *   Key to hash. Must be non-NULL.
- * @param[in] length
- *   Key length in bytes.
- * @param[in] seed
- *   Hash seed.
- * @return
- *   Hash value.
- */
-typedef uint32_t
-(*rte_swx_hash_func_t)(const void *key,
-		       uint32_t length,
-		       uint32_t seed);
-
 /**
  * Pipeline hash function register
  *
@@ -353,6 +352,28 @@ rte_swx_pipeline_hash_func_register(struct rte_swx_pipeline *p,
 				    const char *name,
 				    rte_swx_hash_func_t func);
 
+/*
+ * RSS.
+ */
+
+/**
+ * Pipeline Receive Side Scaling (RSS) object configure
+ *
+ * @param[in] p
+ *   Pipeline handle.
+ * @param[in] name
+ *   Name for the new RSS object.
+ * @return
+ *   0 on success or the following error codes otherwise:
+ *   -EINVAL: Invalid argument;
+ *   -ENOMEM: Not enough space/cannot allocate memory;
+ *   -EEXIST: RSS object with this name already exists.
+ */
+__rte_experimental
+int
+rte_swx_pipeline_rss_config(struct rte_swx_pipeline *p,
+			    const char *name);
+
 /*
  * Packet headers and meta-data
  */
@@ -683,6 +704,13 @@ struct rte_swx_pipeline_table_params {
 	 * list.
 	 */
 	int default_action_is_const;
+
+	/** Hash function name. When not set to NULL, it must point to one of
+	 * the hash functions that were registered for the current pipeline.
+	 * Ignored by the table implementation when not needed. When needed but
+	 * NULL, the table implementation will select the hash function to use.
+	 */
+	const char *hash_func_name;
 };
 
 /**
@@ -823,6 +851,12 @@ struct rte_swx_pipeline_learner_params {
 	 * list.
 	 */
 	int default_action_is_const;
+
+	/** Hash function name. When not set to NULL, it must point to one of
+	 * the hash functions that were registered for the current pipeline.
+	 * When NULL, the default hash function will be used.
+	 */
+	const char *hash_func_name;
 };
 
 /**
@@ -942,12 +976,12 @@ int
 rte_swx_pipeline_build(struct rte_swx_pipeline *p);
 
 /**
- * Pipeline build from specification file
+ * Pipeline C code generate based on input specification file
  *
- * @param[in] p
- *   Pipeline handle.
- * @param[in] spec
- *   Pipeline specification file.
+ * @param[in] spec_file
+ *   Pipeline specification file (.spec) provided as input.
+ * @param[in] code_file
+ *   Pipeline C language file (.c) to be generated.
  * @param[out] err_line
  *   In case of error and non-NULL, the line number within the *spec* file where
  *   the error occurred. The first line number in the file is 1.
@@ -957,15 +991,48 @@ rte_swx_pipeline_build(struct rte_swx_pipeline *p);
  *   0 on success or the following error codes otherwise:
  *   -EINVAL: Invalid argument;
  *   -ENOMEM: Not enough space/cannot allocate memory;
- *   -EEXIST: Resource with the same name already exists;
+ *   -EEXIST: Resource with the same name already exists.
+ */
+__rte_experimental
+int
+rte_swx_pipeline_codegen(FILE *spec_file,
+			 FILE *code_file,
+			 uint32_t *err_line,
+			 const char **err_msg);
+
+/**
+ * Pipeline build from shared object library
+ *
+ * The shared object library must be built from the C language source code file
+ * previously generated by the rte_swx_pipeline_codegen() API function.
+ *
+ * The pipeline I/O specification file defines the I/O ports of the pipeline.
+ *
+ * @param[out] p
+ *   Pipeline handle. Must point to valid memory. Contains valid pipeline handle
+ *   when the function returns successfully.
+ * @param[in] name
+ *   Pipeline unique name.
+ * @param[in] lib_file_name
+ *   Shared object library file name.
+ * @param[in] iospec_file
+ *   Pipeline I/O specification file.
+ * @param[in] numa_node
+ *   Non-Uniform Memory Access (NUMA) node.
+ * @return
+ *   0 on success or the following error codes otherwise:
+ *   -EINVAL: Invalid argument;
+ *   -ENOMEM: Not enough space/cannot allocate memory;
+ *   -EEXIST: Pipeline with this name already exists;
  *   -ENODEV: Extern object or table creation error.
  */
 __rte_experimental
 int
-rte_swx_pipeline_build_from_spec(struct rte_swx_pipeline *p,
-				 FILE *spec,
-				 uint32_t *err_line,
-				 const char **err_msg);
+rte_swx_pipeline_build_from_lib(struct rte_swx_pipeline **p,
+				const char *name,
+				const char *lib_file_name,
+				FILE *iospec_file,
+				int numa_node);
 
 /**
  * Pipeline run
