@@ -14,7 +14,7 @@
 
 #define ERR_MSG_SIZE		128
 #define MAX_CASE_LINE		15
-#define MAX_LINE_CHAR		204800 /*< max number of characters per line */
+#define MAX_LINE_CHAR		204800 /* max number of characters per line */
 #define MAX_NB_TESTS		10240
 #define DEF_MBUF_SEG_SIZE	(UINT16_MAX - sizeof(struct rte_mbuf) - \
 				RTE_PKTMBUF_HEADROOM)
